@@ -533,7 +533,7 @@ struct gnutls_key_st {
 			uint8_t ap_rms[MAX_HASH_SIZE]; /* resumption_master_secret */
 		} tls13; /* tls1.3 */
 
-		/* Folow the SSL3.0 and TLS1.2 key exchanges */
+		/* Follow the SSL3.0 and TLS1.2 key exchanges */
 		struct {
 			/* For ECDH KX */
 			struct {
@@ -851,6 +851,7 @@ struct record_state_st {
 	unsigned aead_tag_size;
 	unsigned is_aead;
 	uint64_t sequence_number;
+	gnutls_record_encryption_level_t level;
 };
 
 
@@ -1196,6 +1197,11 @@ typedef struct {
 	 */
 	gnutls_certificate_request_t send_cert_req;
 
+	/* callback to print the full path of certificate
+	 * validation to the trusted root.
+	 */
+	gnutls_verify_output_function *cert_output_callback;
+
 	size_t max_handshake_data_buffer_size;
 
 	/* PUSH & PULL functions.
@@ -1226,6 +1232,9 @@ typedef struct {
 	gnutls_handshake_hook_func h_hook;
 	unsigned int h_type;	/* the hooked type */
 	int16_t h_post;		/* whether post-generation/receive */
+	gnutls_handshake_read_func h_read_func;
+	gnutls_handshake_secret_func h_secret_func;
+	gnutls_alert_read_func alert_read_func;
 
 	gnutls_keylog_func keylog_func;
 
@@ -1370,6 +1379,7 @@ typedef struct {
 #define HSK_RECORD_SIZE_LIMIT_RECEIVED (1<<26) /* server: record_size_limit extension was seen but not accepted yet */
 #define HSK_OCSP_REQUESTED (1<<27) /* server: client requested OCSP stapling */
 #define HSK_CLIENT_OCSP_REQUESTED (1<<28) /* client: server requested OCSP stapling */
+#define HSK_SERVER_HELLO_RECEIVED (1<<29) /* client: Server Hello message has been received */
 
 	/* The hsk_flags are for use within the ongoing handshake;
 	 * they are reset to zero prior to handshake start by gnutls_handshake. */
