@@ -851,11 +851,71 @@ static int shorten_clist(gnutls_x509_trust_list_t list,
 	return clist_size;
 }
 
-static
-int trust_list_get_issuer(gnutls_x509_trust_list_t list,
-				      gnutls_x509_crt_t cert,
-				      gnutls_x509_crt_t * issuer,
-				      unsigned int flags)
+/* Takes a subject certificate, retrieves a chain from its issuers in
+ * @certificate_list, using the issuer callback set for @list.
+ *
+ * Returns the new size of the list or a negative number on error.
+ */
+static int
+retrieve_issuers(gnutls_x509_trust_list_t list,
+		 gnutls_x509_crt_t subject,
+		 gnutls_x509_crt_t *certificate_list,
+		 unsigned int clist_size_max)
+{
+	gnutls_x509_crt_t *issuers;
+	unsigned int issuers_size;
+	unsigned int i;
+	int ret;
+
+	if (!list->issuer_callback) {
+		return 0;
+	}
+
+	_gnutls_cert_log("calling issuer callback on", subject);
+
+	ret = list->issuer_callback(list, subject, &issuers, &issuers_size);
+	if (ret < 0) {
+		return gnutls_assert_val(ret);
+	}
+
+	/* Ignore empty list */
+	if (!issuers_size) {
+		ret = 0;
+		goto cleanup;
+	}
+
+	if (issuers_size > clist_size_max) {
+		_gnutls_debug_log("too many issuers returned; skipping\n");
+		ret = 0;
+		goto cleanup;
+	}
+
+	for (i = 0; i < issuers_size; i++) {
+		if (!gnutls_x509_crt_check_issuer(subject, issuers[i])) {
+			_gnutls_cert_log("unrelated certificate; skipping",
+					 issuers[i]);
+			break;
+		}
+		subject = issuers[i];
+	}
+
+	ret = i;
+
+	memcpy(certificate_list, issuers, ret * sizeof(gnutls_x509_crt_t));
+
+ cleanup:
+	for (i = ret; i < issuers_size; i++) {
+		gnutls_x509_crt_deinit(issuers[i]);
+	}
+	gnutls_free(issuers);
+
+	return ret;
+}
+
+int _gnutls_trust_list_get_issuer(gnutls_x509_trust_list_t list,
+				  gnutls_x509_crt_t cert,
+				  gnutls_x509_crt_t * issuer,
+				  unsigned int flags)
 {
 	int ret;
 	unsigned int i;
@@ -968,7 +1028,7 @@ int gnutls_x509_trust_list_get_issuer(gnutls_x509_trust_list_t list,
 {
 	int ret;
 
-	ret = trust_list_get_issuer(list, cert, issuer, flags);
+	ret = _gnutls_trust_list_get_issuer(list, cert, issuer, flags);
 	if (ret == 0) {
 		return 0;
 	}
@@ -1249,18 +1309,20 @@ gnutls_x509_trust_list_verify_crt(gnutls_x509_trust_list_t list,
  **/
 int
 gnutls_x509_trust_list_verify_crt2(gnutls_x509_trust_list_t list,
-				  gnutls_x509_crt_t * cert_list,
-				  unsigned int cert_list_size,
-				  gnutls_typed_vdata_st *data,
-				  unsigned int elements,
-				  unsigned int flags,
-				  unsigned int *voutput,
-				  gnutls_verify_output_function func)
+				   gnutls_x509_crt_t * cert_list,
+				   unsigned int cert_list_size,
+				   gnutls_typed_vdata_st *data,
+				   unsigned int elements,
+				   unsigned int flags,
+				   unsigned int *voutput,
+				   gnutls_verify_output_function func)
 {
-	int ret;
+	int ret = 0;
 	unsigned int i;
 	size_t hash;
 	gnutls_x509_crt_t sorted[DEFAULT_MAX_VERIFY_DEPTH];
+	gnutls_x509_crt_t retrieved[DEFAULT_MAX_VERIFY_DEPTH];
+	unsigned int retrieved_size = 0;
 	const char *hostname = NULL, *purpose = NULL, *email = NULL;
 	unsigned hostname_size = 0;
 	unsigned have_set_name = 0;
@@ -1311,8 +1373,57 @@ gnutls_x509_trust_list_verify_crt2(gnutls_x509_trust_list_t list,
 		}
 	}
 
-	if (!(flags & GNUTLS_VERIFY_DO_NOT_ALLOW_UNSORTED_CHAIN))
-		cert_list = _gnutls_sort_clist(sorted, cert_list, &cert_list_size, NULL);
+	memcpy(sorted, cert_list, cert_list_size * sizeof(gnutls_x509_crt_t));
+	cert_list = sorted;
+
+	for (i = 0; i < cert_list_size &&
+		     cert_list_size <= DEFAULT_MAX_VERIFY_DEPTH; i++) {
+		if (!(flags & GNUTLS_VERIFY_DO_NOT_ALLOW_UNSORTED_CHAIN)) {
+			unsigned int sorted_size;
+
+			sorted_size = _gnutls_sort_clist(&cert_list[i],
+							 cert_list_size - i);
+			i += sorted_size - 1;
+		}
+
+		if (i == cert_list_size - 1) {
+			gnutls_x509_crt_t issuer;
+
+			/* If it is the last certificate and its issuer is
+			 * known, don't need to run issuer callback. */
+			if (_gnutls_trust_list_get_issuer(list,
+							  cert_list[i],
+							  &issuer,
+							  0) == 0) {
+				break;
+			}
+		} else if (gnutls_x509_crt_check_issuer(cert_list[i],
+							cert_list[i + 1])) {
+			/* There is no gap between this and the next
+			 * certificate. */
+			continue;
+		}
+
+		ret = retrieve_issuers(list,
+				       cert_list[i],
+				       &retrieved[retrieved_size],
+				       DEFAULT_MAX_VERIFY_DEPTH -
+				       MAX(retrieved_size,
+					   cert_list_size));
+		if (ret < 0) {
+			break;
+		} else if (ret > 0) {
+			memmove(&cert_list[i + 1 + ret],
+				&cert_list[i + 1],
+				(cert_list_size - i - 1) *
+				sizeof(gnutls_x509_crt_t));
+			memcpy(&cert_list[i + 1],
+			       &retrieved[retrieved_size],
+			       ret * sizeof(gnutls_x509_crt_t));
+			retrieved_size += ret;
+			cert_list_size += ret;
+		}
+	}
 
 	cert_list_size = shorten_clist(list, cert_list, cert_list_size);
 	if (cert_list_size <= 0)
@@ -1331,15 +1442,15 @@ gnutls_x509_trust_list_verify_crt2(gnutls_x509_trust_list_t list,
 		*voutput = 0;
 		*voutput |= GNUTLS_CERT_REVOKED;
 		*voutput |= GNUTLS_CERT_INVALID;
-		return 0;
+		ret = 0;
+		goto cleanup;
 	}
 
 	*voutput =
-	    _gnutls_verify_crt_status(cert_list, cert_list_size,
-					    list->node[hash].trusted_cas,
-					    list->
-					    node[hash].trusted_ca_size,
-					    flags, purpose, func);
+	    _gnutls_verify_crt_status(list, cert_list, cert_list_size,
+				      list->node[hash].trusted_cas,
+				      list->node[hash].trusted_ca_size,
+				      flags, purpose, func);
 	saved_output = *voutput;
 
 	if (SIGNER_OLD_OR_UNKNOWN(*voutput) &&
@@ -1357,11 +1468,10 @@ gnutls_x509_trust_list_verify_crt2(gnutls_x509_trust_list_t list,
 		 _gnutls_debug_log("issuer in verification was not found or insecure; trying against trust list\n");
 
 		*voutput =
-		    _gnutls_verify_crt_status(cert_list, cert_list_size,
-					    list->node[hash].trusted_cas,
-					    list->
-					    node[hash].trusted_ca_size,
-					    flags, purpose, func);
+		    _gnutls_verify_crt_status(list, cert_list, cert_list_size,
+					      list->node[hash].trusted_cas,
+					      list->node[hash].trusted_ca_size,
+					      flags, purpose, func);
 		if (*voutput != 0) {
 			if (SIGNER_WAS_KNOWN(saved_output))
 				*voutput = saved_output;
@@ -1375,10 +1485,10 @@ gnutls_x509_trust_list_verify_crt2(gnutls_x509_trust_list_t list,
 	if (SIGNER_OLD_OR_UNKNOWN(*voutput) && list->pkcs11_token) {
 		/* use the token for verification */
 
-		*voutput = _gnutls_pkcs11_verify_crt_status(list->pkcs11_token,
-								cert_list, cert_list_size,
-								purpose,
-								flags, func);
+		*voutput = _gnutls_pkcs11_verify_crt_status(list, list->pkcs11_token,
+							    cert_list, cert_list_size,
+							    purpose,
+							    flags, func);
 		if (*voutput != 0) {
 			if (SIGNER_WAS_KNOWN(saved_output))
 				*voutput = saved_output;
@@ -1425,8 +1535,10 @@ gnutls_x509_trust_list_verify_crt2(gnutls_x509_trust_list_t list,
 
 	/* CRL checks follow */
 
-	if (*voutput != 0 || (flags & GNUTLS_VERIFY_DISABLE_CRL_CHECKS))
-		return 0;
+	if (*voutput != 0 || (flags & GNUTLS_VERIFY_DISABLE_CRL_CHECKS)) {
+		ret = 0;
+		goto cleanup;
+	}
 
 	/* Check revocation of individual certificates.
 	 * start with the last one that we already have its hash
@@ -1440,7 +1552,8 @@ gnutls_x509_trust_list_verify_crt2(gnutls_x509_trust_list_t list,
 	if (ret == 1) {		/* revoked */
 		*voutput |= GNUTLS_CERT_REVOKED;
 		*voutput |= GNUTLS_CERT_INVALID;
-		return 0;
+		ret = 0;
+		goto cleanup;
 	}
 
 	for (i = 0; i < cert_list_size - 1; i++) {
@@ -1459,11 +1572,16 @@ gnutls_x509_trust_list_verify_crt2(gnutls_x509_trust_list_t list,
 		} else if (ret == 1) {	/* revoked */
 			*voutput |= GNUTLS_CERT_REVOKED;
 			*voutput |= GNUTLS_CERT_INVALID;
-			return 0;
+			ret = 0;
+			goto cleanup;
 		}
 	}
 
-	return 0;
+ cleanup:
+	for (i = 0; i < retrieved_size; i++) {
+		gnutls_x509_crt_deinit(retrieved[i]);
+	}
+	return ret;
 }
 
 /**
