@@ -20,9 +20,9 @@
 # along with GnuTLS; if not, write to the Free Software Foundation,
 # Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
-srcdir="${srcdir:-.}"
-CERTTOOL="${CERTTOOL:-../src/certtool${EXEEXT}}"
-TPMTOOL="${TPMTOOL:-../src/tpmtool${EXEEXT}}"
+: ${srcdir=.}
+: ${CERTTOOL=../src/certtool${EXEEXT}}
+: ${TPMTOOL=../src/tpmtool${EXEEXT}}
 
 if [ "$(id -u)" -ne 0 ]; then
 	echo "Need to be root to run this test."
@@ -138,7 +138,6 @@ start_tcsd()
 	local tcsd_conf=$workdir/tcsd.conf
 	local tcsd_system_ps_file=$workdir/system_ps_file
 	local tcsd_pidfile=$workdir/tcsd.pid
-	local owner
 
 	start_swtpm "$workdir"
 	[ $? -ne 0 ] && return 1
@@ -147,36 +146,20 @@ start_tcsd()
 port = $TCSD_LISTEN_PORT
 system_ps_file = $tcsd_system_ps_file
 _EOF_
-	# older versions of trousers require tss:tss ownership of the
-	# config file, later ones root:tss
-	for owner in tss root; do
-		if [ "$owner" = "tss" ]; then
-			chmod 0600 $tcsd_conf
-		else
-			chmod 0640 $tcsd_conf
-		fi
-		chown $owner:tss $tcsd_conf
 
-		bash -c "TCSD_USE_TCP_DEVICE=1 TCSD_TCP_DEVICE_PORT=$SWTPM_SERVER_PORT tcsd -c $tcsd_conf -e -f &>/dev/null & echo \$! > $tcsd_pidfile; wait" &
-		BASH_PID=$!
+	chown tss:tss $tcsd_conf
+	chmod 0600 $tcsd_conf
 
-		if wait_for_file $tcsd_pidfile 3; then
-			echo "Could not get TCSD's PID file"
-			return 1
-		fi
+	bash -c "TCSD_USE_TCP_DEVICE=1 TCSD_TCP_DEVICE_PORT=$SWTPM_SERVER_PORT tcsd -c $tcsd_conf -e -f &>/dev/null & echo \$! > $tcsd_pidfile; wait" &
+	BASH_PID=$!
 
-		sleep 0.5
-		TCSD_PID=$(cat $tcsd_pidfile)
-		kill -0 "${TCSD_PID}"
-		if [ $? -ne 0 ]; then
-			# Try again with other owner
-			continue
-		fi
-		return 0
-	done
+	if wait_for_file $tcsd_pidfile 3; then
+		echo "Could not get TCSD's PID file"
+		return 1
+	fi
 
-	echo "TCSD could not be started"
-	return 1
+	TCSD_PID=$(cat $tcsd_pidfile)
+	return 0
 }
 
 stop_tcsd()
